@@ -22,31 +22,36 @@ For using the MaestroRunner class:
     https://www.schrodinger.com/kb/1842 for details.
 
 """
-import time
 import os
 import re
 import shutil
 import signal
 import subprocess
-from typing import Optional
+import time
+from dataclasses import dataclass
 from string import Template
+from typing import Optional, Union
 from urllib.parse import quote
+
 import numpy as np
 import pubchempy as pcp
-from pymatgen.io.lammps.data import LammpsData
+from monty.json import MSONable
+from monty.serialization import loadfn
+from pymatgen.core import Lattice, Structure
+from pymatgen.core.ion import Ion
+from pymatgen.io.lammps.data import ForceField, LammpsData, Topology, lattice_2_lmpbox
 from selenium import webdriver
-from selenium.webdriver.common.by import By
-from selenium.webdriver.support.ui import WebDriverWait
-from selenium.webdriver.support import expected_conditions as EC
 from selenium.common.exceptions import (
-    TimeoutException,
     NoSuchElementException,
+    TimeoutException,
     WebDriverException,
 )
-from typing_extensions import Final
-
-from mdgo.util import lmp_mass_to_name, ff_parser, sdf_to_pdb
+from selenium.webdriver.common.by import By
+from selenium.webdriver.support import expected_conditions as EC
+from selenium.webdriver.support.ui import WebDriverWait
+from typing_extensions import Final, Literal
 
+from mdgo.util import ff_parser, lmp_mass_to_name, sdf_to_pdb
 
 __author__ = "Tingzheng Hou"
 __version__ = "1.0"
@@ -64,18 +69,26 @@ DATA_MODELS: Final[dict] = {
         "spc": "water_spc.lmp",
         "spce": "water_spce.lmp",
         "tip3pew": "water_tip3p_ew.lmp",
+        "tip3pfb": "water_tip3p_fb.lmp",
+        "opc3": "water_opc3.lmp",
         "tip4p2005": "water_tip4p_2005.lmp",
         "tip4pew": "water_tip4p_ew.lmp",
+        "tip4pfb": "water_tip4p_fb.lmp",
+        "opc": "water_opc.lmp",
     },
-    "ion": {
-        "aq": ["default"],
-        "aqvist": ["default"],
-        "jj": ["default"],
-        "jensen_jorgensen": ["default"],
-        "jc": ["spce", "tip3p", "tip4pew"],
-        "joung_cheatham": ["spce", "tip3p", "tip4pew"],
-    },
-    "alias": {"aq": "aqvist", "jj": "jensen_jorgensen", "jc": "joung_cheatham"},
+}
+WATER_SIGMA: Final[dict] = {
+    "spc": 3.16557,
+    "spce": 3.16557,
+    "tip3p": 3.1507,
+    "tip3pew": 3.188,
+    "tip3pfb": 3.178,
+    "opc": 3.16655,
+    "opc3": 3.17427,
+    "tip4p": 3.1536,
+    "tip4p2005": 3.1589,
+    "tip4pew": 3.16435,
+    "tip4pfb": 3.1655,
 }
 
 
@@ -213,7 +226,7 @@ class FFcrawler:
             data_obj = LammpsData.from_file(lmp_file)
             element_id_dict = lmp_mass_to_name(data_obj.masses)
             coords = data_obj.atoms[["type", "x", "y", "z"]]
-            lines = list()
+            lines = []
             lines.append(str(len(coords.index)))
             lines.append("")
             for _, r in coords.iterrows():
@@ -341,7 +354,7 @@ class MaestroRunner:
                 preexec_fn=os.setsid,
             )
         except subprocess.CalledProcessError as e:
-            raise ValueError("Maestro failed with errorcode {}  and stderr: {}".format(e.returncode, e.stderr)) from e
+            raise ValueError(f"Maestro failed with errorcode {e.returncode}  and stderr: {e.stderr}") from e
 
         counter = 0
         while not os.path.isfile(self.mae + ".mae"):
@@ -349,7 +362,7 @@ class MaestroRunner:
             counter += 1
             if counter > wait:
                 os.killpg(os.getpgid(p.pid), signal.SIGTERM)
-                raise TimeoutError("Failed to generate Maestro file in {} secs!".format(wait))
+                raise TimeoutError(f"Failed to generate Maestro file in {wait} secs!")
         print("Maestro file generated!")
         os.killpg(os.getpgid(p.pid), signal.SIGTERM)
 
@@ -364,7 +377,7 @@ class MaestroRunner:
                 stderr=subprocess.PIPE,
             )
         except subprocess.CalledProcessError as e:
-            raise ValueError("Maestro failed with errorcode {} and stderr: {}".format(e.returncode, e.stderr)) from e
+            raise ValueError(f"Maestro failed with errorcode {e.returncode} and stderr: {e.stderr}") from e
         print("Maestro force field file generated.")
         if self.out:
             if self.out == "lmp":
@@ -560,10 +573,65 @@ class PubChemRunner:
         return cid
 
 
+@dataclass
+class IonLJData(MSONable):
+    """
+    A lightweight dataclass for storing ion force field parameters. The data
+    file ion_lj_params.json distributed with mdgo is a serialized list of these
+    objects.
+
+    Attributes:
+        name: The name of the parameter set
+        formula: formula of the ion, e.g. 'Li+'
+        combining_rule: the method used to compute pairwise interaction parameters
+            from single atom parameters. 'geometric' or 'LB' for Lorentz-Berthelot
+        water_model: The water model for which the ion parameters were optimized.
+        sigma: The Lennard Jones sigma value, in Å
+        epsilon: The Lennard Jones epsilon value, in kcal/mol
+    """
+
+    name: Literal["jensen_jorgensen", "joung_cheatham", "li_merz"]
+    formula: str
+    combining_rule: Literal["geometric", "LB"]
+    water_model: Literal[
+        "spc",
+        "spce",
+        "tip3p",
+        "tip3pew",
+        "tip3pfb",
+        "opc3",
+        "tip4p2005",
+        "tip4p",
+        "tip4pew",
+        "tip4pfb",
+        "opc",
+    ]
+    sigma: float
+    epsilon: float
+
+
 class Aqueous:
     """
     A class for retreiving water and ion force field parameters.
 
+    Available water models are:
+        1. SPC
+        2. SPC/E
+        3. TIP3P-EW
+        4. TIP3P-FB
+        5. OPC3
+        6. TIP4P-EW
+        7. TIP4P-2005
+        8. TIP4P-FB
+        9. OPC
+
+    Multiple sets of Lennard Jones parameters for ions are available as well.
+    Not every set is available for every water model. The parameter sets included
+    are:
+        1. Jensen and Jorgensen, 2006 (abbreviation 'jj')
+        2. Joung and Cheatham, 2008 (abbreviation 'jc')
+        3. Li and Merz group, 2020 (abbreviation, 'lm')
+
     Examples:
         Retreive SPC/E water model:
         >>> spce_data = Aqueous.get_water()
@@ -574,64 +642,183 @@ class Aqueous:
     """
 
     @staticmethod
-    def get_water(model: str = "spce") -> Optional[LammpsData]:
+    def get_water(model: str = "spce") -> LammpsData:
         """
         Retrieve water model parameters.
 
         Args:
-            model: Water model to use. Valid choices are "spc", "spce",
-                "tip3pew", "tip4p2005", and "tip4pew". (Default: "spce")
+            model: Water model to use. Valid choices are "spc", "spce", "opc3",
+                "tip3pew", "tip3pfb", "tip4p2005", "tip4pew", "tip4pfb", and "opc".
+                (Default: "spce")
         Returns:
             LammpsData: Force field parameters for the chosen water model.
                 If you specify an invalid water model, None is returned.
         """
-        data_path = DATA_DIR
         signature = "".join(re.split(r"[\W|_]+", model)).lower()
         if DATA_MODELS["water"].get(signature):
-            return LammpsData.from_file(os.path.join(data_path, "water", DATA_MODELS["water"].get(signature)))
-        print("Water model not found. Please specify a customized data path or try another water model.\n")
-        return None
+            return LammpsData.from_file(os.path.join(DATA_DIR, "water", DATA_MODELS["water"].get(signature)))
+        raise ValueError("Water model not found. Please specify a customized data path or try another water model.\n")
 
     @staticmethod
-    def get_ion(model: str = "jensen_jorgensen", water: str = "default", ion: str = "li+") -> Optional[LammpsData]:
+    def get_ion(
+        ion: Union[Ion, str],
+        parameter_set: str = "auto",
+        water_model: str = "auto",
+        mixing_rule: Optional[str] = None,
+    ) -> LammpsData:
         """
         Retrieve force field parameters for an ion in water.
 
         Args:
-            model: Force field to use. Valid choices are "aqvist" (or "aq"),
-                "jensen_jorgensen" (or "jj"), ""joung_cheatham" (or "jc").
-            water: Water model to use. For the jensen_jorgensen and aqvist
-                models, the only choice is 'default'. For the joung_cheatham
-                model, valid choices are "spce", "tip3p", and "tip4pew".
-            ion: Formula of the ion (e.g., "Li+").
+            ion: Formula of the ion (e.g., "Li+"). Not case sensitive. May be
+                passed as either a string or an Ion object.
+            parameter_set: Force field parameters to use for ions.
+                Valid choices are:
+                    1. "jj" for the Jensen and Jorgensen parameters (2006)"
+                    2. "jc" for Joung-Cheatham parameters (2008)
+                    3. "lm" for the Li and Merz group parameters (2020-2021)"
+                The default parameter set is "auto", which assigns a recommended
+                parameter set that is compatible with the chosen water model.
+            water_model: Water model to use. Models must be given as a string
+                (not case sensitive). "-" and "/" are ignored. Hence "tip3pfb"
+                and "TIP3P-FB" are both valid inputs for the TIP3P-FB water model.
+                Available water models are:
+                    1. SPC
+                    2. SPC/E
+                    3. TIP3P-EW
+                    4. TIP3P-FB
+                    5. OPC3
+                    6. TIP4P-EW
+                    7. TIP4P-2005
+                    8. TIP4P-FB
+                    9. OPC
+                The default water model is "auto", which assigns a recommended
+                water model that is compatible with the chosen ion parameters. Other
+                combinations are possible at your own risk. See documentation.
+
+            When both the parameter_set and water_model are set to "auto", the function returns the
+            Joung-Cheatham parameters for the SPC/E water model.
+
+                For a systematic comparison of the performance of different water models, refer to
+
+                    Sachini et al., Systematic Comparison of the Structural and Dynamic Properties of
+                    Commonly Used Water Models for Molecular Dynamics Simulations. J. Chem. Inf. Model.
+                    2021, 61, 9, 4521–4536. https://doi.org/10.1021/acs.jcim.1c00794
+
+            mixing_rule: The mixing rule to use for the ion parameter. Default to None, which does not
+                change the original mixing rule of the parameter set. Available choices are 'LB'
+                (Lorentz-Berthelot or arithmetic) and 'geometric'. If the specified mixing rule does not
+                match the default mixing rule of the parameter set, the output parameter will be converted
+                accordingly.
+
 
         Returns:
             Force field parameters for the chosen water model.
-            If the desired combination of force field and water model
-            for the given ion is not available, None is returned.
         """
-        data_path = DATA_DIR
-        alias = DATA_MODELS.get("alias", {})
-        signature = model.lower()
-        if signature in alias:
-            signature = alias.get(model)
-        ion_type = ion.capitalize()
-        for key in DATA_MODELS["ion"]:
-            if key.startswith(signature):
-                ion_model = DATA_MODELS["ion"].get(key)
-                if water in ion_model:
-                    if water == "default":
-                        file_path = os.path.join(data_path, "ion", key, ion_type + ".lmp")
-                    else:
-                        file_path = os.path.join(data_path, "ion", key, water, ion_type + ".lmp")
-                    if os.path.exists(file_path):
-                        return LammpsData.from_file(file_path)
-                    print("Ion not found. Please try another ion.\n")
-                    return None
-                print("Water model not found. Please try another water model.\n")
-                return None
-        print("Ion model not found. Please try another ion model.\n")
-        return None
+        alias = {"aq": "aqvist", "jj": "jensen_jorgensen", "jc": "joung_cheatham", "lm": "li_merz"}
+        default_sets = {
+            "spc": "N/A",
+            "spce": "jc",
+            "tip3p": "jc",
+            "tip3pew": "N/A",
+            "tip3pfb": "lm",
+            "opc3": "lm",
+            "tip4p2005": "N/A",
+            "tip4p": "jj",
+            "tip4pew": "jc",
+            "tip4pfb": "lm",
+            "opc": "lm",
+            "jj": "tip4p",
+            "jc": "spce",
+            "lm": "tip4pfb",
+        }
+        water_model = water_model.replace("-", "").replace("/", "").lower()
+        parameter_set = parameter_set.lower()
+
+        if water_model == "auto" and parameter_set == "auto":
+            water_model = "spce"
+            parameter_set = "jc"
+        elif parameter_set == "auto":
+            parameter_set = default_sets.get(water_model, parameter_set)
+            if parameter_set == "N/A":
+                raise ValueError(
+                    f"The {water_model} water model has no specifically parameterized ion parameter sets"
+                    "Please try a different water model."
+                )
+        elif water_model == "auto":
+            water_model = default_sets.get(parameter_set, water_model)
+
+        parameter_set = alias.get(parameter_set, parameter_set)
+
+        # Make the Ion object to get mass and charge
+        if isinstance(ion, Ion):
+            ion_obj = ion
+        else:
+            ion_obj = Ion.from_formula(ion.capitalize())
+
+        # load ion data as a list of IonLJData objects
+        ion_data = loadfn(os.path.join(DATA_DIR, "ion_lj_params.json"))
+
+        # make sure the ion is in the DataFrame
+        key = ion_obj.reduced_formula
+        filtered_data = [d for d in ion_data if d.formula == key]
+        if len(filtered_data) == 0:
+            raise ValueError(f"Ion {key} not found in database. Please try a different ion.")
+
+        # make sure the parameter set is in the DataFrame
+        filtered_data = [d for d in filtered_data if d.name == parameter_set and d.water_model == water_model]
+        if len(filtered_data) == 0:
+            raise ValueError(
+                f"No {parameter_set} parameters for water model {water_model} for ion {key}. "
+                "See documentation and try a different combination."
+            )
+
+        if len(filtered_data) != 1:
+            raise ValueError(
+                f"Something is wrong: multiple ion data entries for {key}, {parameter_set}, and {water_model}"
+            )
+
+        # we only consider monatomic ions at present
+        # construct a cubic LammpsBox from a lattice
+        lat = Lattice([[1, 0, 0], [0, 1, 0], [0, 0, 1]])
+        box = lattice_2_lmpbox(lat)[0]
+        # put it in the center of a cubic Structure
+        struct = Structure(lat, ion_obj, [[0.5, 0.5, 0.5]])
+        # construct Topology with the ion centered in the box
+        topo = Topology(struct, charges=[ion_obj.charge])
+
+        # retrieve Lennard-Jones parameters
+        # construct ForceField object
+        sigma = filtered_data[0].sigma
+        epsilon = filtered_data[0].epsilon
+        if mixing_rule is None:
+            pass
+        else:
+            default_mixing = filtered_data[0].combining_rule
+            water_sigma = WATER_SIGMA.get(filtered_data[0].water_model)
+            if mixing_rule.lower() in ["lb", "arithmetic", "lorentz-berthelot", "lorentz berthelot"]:
+                mixing_rule = "LB"
+            elif mixing_rule.lower() == "geometric":
+                mixing_rule = "geometric"
+            else:
+                raise ValueError("Invalid mixing rule. Supported mixing rules are 'LB'(arithmetic) and 'geometric'. ")
+            if default_mixing == mixing_rule:
+                pass
+            elif default_mixing == "LB" and mixing_rule == "geometric":
+                sigma = ((water_sigma + sigma) / 2) ** 2 / water_sigma
+                print(
+                    "The parameter mixing rule has been converted from the original 'LB' to 'geometric'.\n"
+                    "Please use the parameter set with caution!"
+                )
+            else:
+                sigma = 2 * ((water_sigma * sigma) ** (1 / 2)) - water_sigma
+                print(
+                    "The parameter mixing rule has been converted from the original 'geometric' to 'LB'.\n"
+                    "Please use the parameter set with caution!"
+                )
+        ff = ForceField([(str(e), e) for e in ion_obj.elements], nonbond_coeffs=[[epsilon, sigma]])
+
+        return LammpsData.from_ff_and_topologies(box, ff, [topo], atom_style="full")
 
 
 class ChargeWriter:
@@ -661,7 +848,7 @@ class ChargeWriter:
         Returns:
             A recreated LammpsData obj
         """
-        items = dict()
+        items = {}
         items["box"] = self.data.box
         items["masses"] = self.data.masses
         atoms = self.data.atoms.copy(deep=True)
@@ -695,7 +882,7 @@ class ChargeWriter:
         number_str = repr(float(number))
         tokens = number_str.split(".")
         if len(tokens) > 2:
-            raise ValueError("Invalid number '{}' only 1 decimal allowed".format(number))
+            raise ValueError(f"Invalid number '{number}' only 1 decimal allowed")
         if len(tokens) == 2:
             decimal_num = tokens[1][: self.precision].rstrip("0")
             return len(decimal_num)
